﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HeadTailSocketFW
{
    public class ByteQueue
    {
        private object _lock = new object();

        private Queue<byte> _queue;

        public int Count { get { return _queue.Count; } }

        public ByteQueue(int capacity)
        {
            lock (_lock)
            {
                _queue = new Queue<byte>(capacity);
            }
        }

        public void Enqueue(byte item)
        {
            lock (_lock)
            {
                _queue.Enqueue(item);
            }
        }

        public void Enqueue(byte[] array)
        {
            lock (_lock)
            {
                for (int i = 0; i < array.Length; i++)
                {
                    _queue.Enqueue(array[i]);
                }
            }
        }

        public byte Dequeue()
        {
            lock (_lock)
            {
                return _queue.Dequeue();
            }
        }

        public byte[] Dequeue(int size)
        {
            lock (_lock)
            {
                byte[] array = new byte[size];
                for (int i = 0; i < array.Length; i++)
                {
                    array[i] = _queue.Dequeue();
                }

                return array;
            }
        }

        public int FindIndex(byte[] item, int startIndex = 0)
        {
            if (item == null || item.Length < 1)
                return -1;
            byte[] queueBytes = new byte[_queue.Count];
            _queue.CopyTo(queueBytes, 0);
            for (int i = startIndex; i < queueBytes.Length - item.Length + 1; i++)
            {
                if (queueBytes.Skip(i).Take(item.Length).SequenceEqual(item))
                    return i;
            }

            return -1;
        }

        public void Clear()
        {
            lock (_lock)
            {
                _queue.Clear();
            }
        }
    }
}
