﻿using System;
using System.Collections.Generic;
using System.Text;

namespace HeadTailSocketFW
{
    public class HeadTailSocket : ClientSocket
    {
        public byte[] Head { get; set; }
        public byte[] Tail { get; set; }

        public bool IsIncludeHeadTail { get; set; }

        protected override void Interpret()
        {
            // Both head and tail are not exist.
            if ((Head == null || Head.Length < 1) && (Tail == null || Tail.Length < 1))
            {
                RaiseReceivedEvent(this, _buffer.Dequeue(_buffer.Count));
                return;
            }

            byte[] head;
            byte[] tail;

            // Head only
            if ((Head != null || Head.Length > 0) && (Tail == null || Tail.Length < 1))
            {
                head = Head;
                tail = Head;
            }


            // Tail only
            if ((Head != null || Head.Length > 0) && (Tail == null || Tail.Length < 1))
            {
                head = Tail;
                tail = Tail;
            }

            // Both
            {
                head = Head;
                tail = Tail;
            }

            int startIndex = _buffer.FindIndex(head);
            if (startIndex < 0)
                return;

            int endIndex = _buffer.FindIndex(tail, startIndex + 1);
            if (endIndex < 0)
                return;

            if (startIndex >= endIndex)
                return;

            byte[] rawPacket = _buffer.Dequeue(endIndex + 1);
            byte[] packet = new byte[endIndex - startIndex - 1];
            Array.Copy(rawPacket, startIndex + 1, packet, 0, packet.Length);
            RaiseReceivedEvent(this, packet);
        }
    }
}
